#! /usr/bin/env python
# -*- coding: koi8-r -*-

__author__ = "Oleg Broytman <phd@phdru.name>"
__copyright__ = "Copyright (C) 2006-2017 PhiloSoft Design"

import sys, os
from Cheetah.Template import Template
from Cheetah.compat import string_type
from blog_db import blog_root, load_blog, save_blog


old_blog = load_blog()

# blog is a dictionary mapping
# (year, month, day) => [list of (file, title, lead, tags)]

blog = {}
years = {}

# bodies is a dictionary mapping file => body

bodies = {}

# Walk the directory recursively
for dirpath, dirs, files in os.walk(blog_root):
   d = os.path.basename(dirpath)
   if not d.startswith("20") and not d.isdigit():
      continue
   for file in files:
      if not file.endswith(".tmpl"):
         continue
      fullpath = os.path.join(dirpath, file)
      template = Template(file=fullpath)
      title_parts = template.Title.decode('utf-8').encode('koi8-r').split()
      title = ' '.join(title_parts[6:])
      lead = template.Lead.decode('utf-8').encode('koi8-r')

      tags = template.Tag
      if isinstance(tags, string_type):
         tags = (tags,)
      tags = [tag.decode('utf-8').encode('koi8-r') for tag in tags]

      if title:
         key = year, month, day = tuple(dirpath[len(blog_root):].split(os.sep)[1:])
         if key in blog:
            days = blog[key]
         else:
            days = blog[key] = []
         days.append((file, title, lead, tags))

         if year in years:
            months = years[year]
         else:
            months = years[year] = {}

         if month in months:
            days = months[month]
         else:
            days = months[month] = []

         if day not in days: days.append(day)

         file = file[:-len("tmpl")] + "html"
         key = (year, month, day, file)
         body = template.body()
         if isinstance(body, unicode):
            body = body.encode('koi8-r')
         bodies[key] = body

# Need to save the blog?
if blog != old_blog:
    save_blog(blog)

# Localized month names

import locale
locale.setlocale(locale.LC_ALL, "ru_RU.KOI8-R")
from calendar import _localized_month

locale.setlocale(locale.LC_TIME, 'C')
months_names_en = list(_localized_month('%B'))
months_abbrs_en = list(_localized_month('%b'))

locale.setlocale(locale.LC_TIME, "ru_RU.KOI8-R")
# months_names_ru = list(_localized_month('%B'))

months_names_ru = ['', "", "", "", "", "", "",
   "", "", "", "", "", ""
]

months_names_ru0 = ['', "", "", "", "", "", "",
   "", "", "", "", "", ""
]

from news import write_if_changed


def write_template(level, year, month, day, titles, tags=None):
   path = [blog_root]
   if level >= 1:
      path.append(year)
   if level >= 2:
      path.append(month)
   if level == 3:
      path.append(day)
   path.append("index.tmpl")
   index_name = os.path.join(*path)

   new_text = ["""\
## THIS FILE IS AUTOMATICALLY GENERATED. DO NOT EDIT.
#encoding koi8-r
#extends phd_site
#implements respond
"""]

   if level == 0:
      new_text.append("""\
#attr $Title = "Oleg Broytman's blog"
#attr $Description = "Broytman Russian Blog Index Document"
#attr $Copyright = %(cyear)s
#attr $alternates = ((" [Atom 1.0]  ", "application/atom+xml", "atom_10_titles.xml"),
                     (" [Atom 1.0]", "application/atom+xml", "atom_10.xml"),
                     (" [Atom 1.0]  ", "application/atom+xml", "atom_10_full.xml"),
                     (" [RSS 2.0]  ",  "application/rss+xml",  "rss_20_titles.xml"),
                     (" [RSS 2.0]",  "application/rss+xml",  "rss_20.xml"),
                     (" [RSS 2.0]  ",  "application/rss+xml",  "rss_20_full.xml"),
)
##
#def body_html
<h1></h1>
""" % {"cyear": year or 2005})

   elif level == 1:
      new_text.append("""\
#attr $Title = "Oleg Broytman's blog: %(year)s"
#attr $Description = "Broytman Russian Blog %(year)s Index Document"
#attr $Copyright = %(cyear)s
##
#def body_html
<h1>: %(year)s</h1>
""" % {"year": year, "cyear": year or 2005})

   elif level == 2:
      imonth = int(month)
      new_text.append("""\
#attr $Title = "Oleg Broytman's blog: %(month_abbr_en)s %(year)s"
#attr $Description = "Broytman Russian Blog %(month_name_en)s %(year)s Index Document"
#attr $Copyright = %(cyear)s
##
#def body_html
<h1>: %(month_name_ru0)s %(year)s</h1>
""" % {
      "year": year, "cyear": year or 2005,
      "month_abbr_en": months_abbrs_en[imonth], "month_name_en": months_names_en[imonth],
      "month_name_ru0": months_names_ru0[imonth],
   })

   elif level == 3:
      iday = int(day)
      imonth = int(month)

      if len(titles) == 1:
         new_text.append("""\
#attr $Refresh = "0; URL=%s"
""" % titles[0][3])

      new_text.append("""\
#attr $Title = "Oleg Broytman's blog: %(day)d %(month_abbr_en)s %(year)s"
#attr $Description = "Broytman Russian Blog %(day)d %(month_name_en)s %(year)s Index Document"
#attr $Copyright = %(cyear)s
##
#def body_html
<h1>: %(day)d %(month_name_ru)s %(year)s</h1>
""" % {
      "year": year, "cyear": year or 2005,
      "month_abbr_en": months_abbrs_en[imonth], "month_name_en": months_names_en[imonth],
      "month_name_ru": months_names_ru[imonth],
      "day": iday
   })

   save_titles = titles[:]
   titles.reverse()

   save_date = None
   for year, month, day, file, title, lead in titles:
      href = []
      if level == 0:
         href.append(year)
      if level <= 1:
         href.append(month)
      if level <= 2:
         href.append(day)
      href.append(file)
      href = '/'.join(href)
      if day[0] == '0': day = day[1:]
      if save_date != (year, month, day):
         if level == 0:
            new_text.append('\n<h2>%s %s %s</h2>' % (day, months_names_ru[int(month)], year))
         else:
            new_text.append('\n<h2>%s %s</h2>' % (day, months_names_ru[int(month)]))
         save_date = year, month, day
      new_text.append('''
<p class="head">
   %s<a href="%s">%s</a>.
</p>
''' % (lead+' ' if lead else '', href, title))

   if level == 0:
      new_text.append("""
<hr>

<p class="head">   
<img src="../../Graphics/atom_10.jpg" border=0>
<A HREF="atom_10_titles.xml">Atom 1.0  </A> /
<A HREF="atom_10.xml">Atom 1.0</A> /
<A HREF="atom_10_full.xml">Atom 1.0  </A>
 <img src="../../Graphics/rss_20.jpg" border=0>
<A HREF="rss_20_titles.xml">RSS 2.0  </A> /
<A HREF="rss_20.xml">RSS 2.0</A> /
<A HREF="rss_20_full.xml">RSS 2.0  </A>.
</p>
""")

      years = {}
      for year, month, day, file, title, lead in save_titles:
         years[year] = True
      new_text.append('''
<p class="head"><a href="tags/"></a>:
''')
      first_tag = True
      for count, tag, links in all_tags:
         if first_tag:
            first_tag = False
         else:
            new_text.append(' - ')
         new_text.append("""<a href="tags/%s.html">%s (%d)</a>""" % (tag, tag, count))
      new_text.append('''
</p>
''')

      max_year = int(sorted(years.keys())[-1])
      years = range(max_year, 2005, -1)

      new_text.append('''
<p class="head"> :
''')

      year_counts = {}
      for year, month, day, file, title, lead in all_titles:
         year_counts[year] = 0
      for year, month, day, file, title, lead in all_titles:
         year_counts[year] += 1

      first_year = True
      for year in years:
         if first_year:
            first_year = False
         else:
            new_text.append(' - ')
         new_text.append('<a href="%s/">%s (%d)</a>' % (year, year, year_counts[str(year)]))
      new_text.append('''
</p>
''')

      new_text.append("""
<hr>
<p class="head"><a href="http://phd.livejournal.com/"></a>
""")

   new_text.append("""\
#end def
$phd_site.respond(self)
""")

   write_if_changed(index_name, ''.join(new_text))


all_tags = {}
all_titles = []
all_titles_tags = []

for year in sorted(years.keys()):
   year_titles = []
   months = years[year]
   for month in sorted(months.keys()):
      month_titles = []
      for day in sorted(months[month]):
         day_titles = []
         key = year, month, day
         if key in blog:
            for file, title, lead, tags in blog[key]:
               if file.endswith(".tmpl"): file = file[:-len("tmpl")] + "html"
               value = (year, month, day, file, title, lead)
               all_titles_tags.append((year, month, day, file, title, lead, tags))
               all_titles.append(value)
               year_titles.append(value)
               month_titles.append(value)
               day_titles.append(value)
               for tag in tags:
                  if tag in all_tags:
                     tag_links = all_tags[tag]
                  else:
                     tag_links = all_tags[tag] = []
                  tag_links.append(value)
         write_template(3, year, month, day, day_titles)
      write_template(2, year, month, day, month_titles)
   write_template(1, year, month, day, year_titles)

def by_count_rev_tag_link(t1, t2):
   """Sort all_tags by count in descending order,
   and by tags and links in ascending order
   """
   r = cmp(t1[0], t2[0])
   if r:
      return -r
   return cmp((t1[1], t1[2]), (t2[1], t2[2]))

all_tags = [(len(links), tag, links) for (tag, links) in all_tags.items()]
all_tags.sort(by_count_rev_tag_link)

write_template(0, year, month, day, all_titles[-20:], all_tags)

new_text = ["""\
## THIS FILE IS AUTOMATICALLY GENERATED. DO NOT EDIT.
#encoding koi8-r
#extends phd_site
#implements respond
#attr $Title = "Oleg Broytman's blog: tags"
#attr $Description = "Broytman Russian Blog Tags Index Document"
#attr $Copyright = 2006
##
#def body_html
<h1></h1>

<p class="head small">
      ,  .
 :</p>
<ul class="small">
    <li> -    , ģ    .</li>
    <li> '!' (NOT, not) -  ,     .</li>
    <li> '&amp;' (AND, and) -  ,     .</li>
    <li> '|' (OR, or) -  ,      .</li>
    <li>  : NOT &gt; AND &gt; OR.  '()'   .</li>
</ul>
<p class="small">
 : linux - ģ 
  linux.html; linux&amp;!audio -      
linux    audio; linux and not audio -   .
</p>

<center>
<form method=GET action="../../../cgi-bin/blog-ru/search-tags/search-tags.py">
    <input type=text name=q>
    <input type=submit name=submit value="">
</form>
</center>

<dl>
"""]

for i, (count, tag, links) in enumerate(all_tags):
   new_text.append("""\
   <dt><a href="%s.html">%s (%d)</a></dt>
""" % (tag, tag, count))

   first = all_tags[0][1]
   if i == 0:
      prev = None
   else:
      prev = all_tags[i-1][1]
   if i >= len(all_tags)-1:
      next = None
   else:
      next = all_tags[i+1][1]
   last = all_tags[-1][1]

   tag_text = ["""\
## THIS FILE IS AUTOMATICALLY GENERATED. DO NOT EDIT.
#encoding koi8-r
#extends phd_site
#implements respond
#attr $Title = "Oleg Broytman's blog: tag %s"
#attr $Description = "Broytman Russian Blog Tag %s Index Document"
""" % (tag, tag)]

   tag_text.append("""\
#attr $First = "%s"
""" % first)

   if prev:
      tag_text.append("""\
#attr $Prev = "%s"
""" % prev)

   if next:
      tag_text.append("""\
#attr $Next = "%s"
""" % next)

   tag_text.append("""\
#attr $Last = "%s"
""" % last)

   tag_text.append("""\
#attr $Copyright = 2006
##
#def body_html
<h1>%s</h1>

<ul>
""" % tag)

   count = 0
   for year, month, day, filename, title, lead in reversed(links):
      link = "../%s/%s/%s/%s" % (year, month, day, filename)
      item_text = """<li><a href="%s">%s/%s/%s: %s%s</a></li>""" % (link, year, month, day, lead+' ' if lead else '', title)

      count += 1
      if count <= 5:
         new_text.append("      <dd>%s</dd>\n" % item_text)

      tag_text.append("   %s\n" % item_text)

   tag_text.append("""\
</ul>
#end def
$phd_site.respond(self)
""")
   write_if_changed(os.path.join(blog_root, "tags", tag+".tmpl"), ''.join(tag_text))

new_text.append("""\
</dl>
#end def
$phd_site.respond(self)
""")
write_if_changed(os.path.join(blog_root, "tags", "index.tmpl"), ''.join(new_text))


from HTMLParser import HTMLParseError
import cgi
from urlparse import urljoin
from m_lib.net.www.html import HTMLParser as _HTMLParser

class HTMLDone(Exception): pass


class FirstPHTMLParser(_HTMLParser):
   def __init__(self):
      _HTMLParser.__init__(self)
      self.first_p = None

   def start_p(self, attrs):
      self.accumulator = '<p>'

   def end_p(self):
      self.first_p = self.accumulator + '</p>'
      raise HTMLDone()

def get_first_p(body):
   parser = FirstPHTMLParser()

   try:
      parser.feed(body)
   except (HTMLParseError, HTMLDone):
      pass

   try:
      parser.close()
   except (HTMLParseError, HTMLDone):
      pass

   return parser.first_p


class AbsURLHTMLParser(_HTMLParser):
   def __init__(self, base):
      _HTMLParser.__init__(self)
      self.base = base

   def start_a(self, attrs):
      self.accumulator += '<a'
      for attrname, value in attrs:
         value = cgi.escape(value, True)
         if attrname == 'href':
            self.accumulator += ' href="%s"' % urljoin(self.base, value)
         else:
            self.accumulator += ' %s="%s"' % (attrname, value)
      self.accumulator += '>'

   def end_a(self):
      self.accumulator += '</a>'

   def start_img(self, attrs):
      self.accumulator += '<img'
      for attrname, value in attrs:
         value = cgi.escape(value, True)
         if attrname == 'src':
            self.accumulator += ' src="%s"' % urljoin(self.base, value)
         else:
            self.accumulator += ' %s="%s"' % (attrname, value)
      self.accumulator += '>'

   def end_img(self):
       pass

def absolute_urls(body, base):
   parser = AbsURLHTMLParser(base)

   try:
      parser.feed(body)
   except HTMLParseError:
      pass

   try:
      parser.close()
   except HTMLParseError:
      pass

   return parser.accumulator


from atom_10 import atom_10
from rss_20 import rss_20
from news import NewsItem

if blog_root:
   blog_root_url = blog_root[
        blog_root.find('/htdocs/phdru.name/') + len('/htdocs/phdru.name/'):]
   baseURL = "http://phdru.name/%s/" % blog_root_url
else:
   baseURL = "http://phdru.name/"

items = []
for item in tuple(reversed(all_titles_tags))[:10]:
   year, month, day, file, title, lead, tags = item
   lead = lead.decode('koi8-r').encode('utf-8')
   title = title.decode('koi8-r').encode('utf-8')
   url_path = "%s/%s/%s/%s" % (year, month, day, file)
   item = NewsItem(
      "%s-%s-%s" % (year, month, day),
      "%s%s" % (lead+' ' if lead else '', title),
      url_path)
   items.append(item)
   item.baseURL = baseURL
   item.categoryList = tags
   body = bodies[(year, month, day, file)]
   body = absolute_urls(body, baseURL + url_path)
   item.body = body
   item.excerpt = get_first_p(body)

namespace = {
   "title": "Oleg Broytman's blog",
   "baseURL": baseURL,
   "indexFile": "",
   "description": "",
   "lang": "ru",
   "author": "Oleg Broytman",
   "email": "phd@phdru.name",
   "generator": os.path.basename(sys.argv[0]),
   "posts": items,
}

# For english dates
locale.setlocale(locale.LC_TIME, 'C')

atom_tmpl = unicode(atom_10(searchList=[namespace])).encode('koi8-r')
write_if_changed(os.path.join(blog_root, "atom_10.xml"), atom_tmpl)
rss_tmpl = unicode(rss_20(searchList=[namespace])).encode('koi8-r')
write_if_changed(os.path.join(blog_root, "rss_20.xml"), rss_tmpl)

for item in items:
    item.excerpt = None

atom_tmpl = unicode(atom_10(searchList=[namespace])).encode('koi8-r')
write_if_changed(os.path.join(blog_root, "atom_10_titles.xml"), atom_tmpl)
rss_tmpl = unicode(rss_20(searchList=[namespace])).encode('koi8-r')
write_if_changed(os.path.join(blog_root, "rss_20_titles.xml"), rss_tmpl)

for item in items:
    item.content = item.body

atom_tmpl = unicode(atom_10(searchList=[namespace])).encode('koi8-r')
write_if_changed(os.path.join(blog_root, "atom_10_full.xml"), atom_tmpl)
rss_tmpl = unicode(rss_20(searchList=[namespace])).encode('koi8-r')
write_if_changed(os.path.join(blog_root, "rss_20_full.xml"), rss_tmpl)
